# CSS Typed OM Level 2

## A Collection of Interesting Ideas, 1 January 1970

This version:
  https://drafts.css-houdini.org/css-typed-om-1/
Feedback:
  public-houdini@w3.org with subject line “`[css-typed-om] … message topic …`” (archives)
Editor:
  Tab Atkins-Bittner (Google)
Former Editor:
  Shane Stephens
Issue Tracking:
  GitHub Issues

Copyright © 1970 W3C® (MIT, ERCIM, Keio, Beihang). W3C liability, trademark and document use rules apply.

## Abstract

## Status of this document

do we want a getUsedStyleMap too? Or should it be getResolvedStyleMap? <https://github.com/w3c/css-houdini-drafts/issues/157>

Need to add a section describing unions of types. <https://github.com/w3c/css-houdini-drafts/issues/163>

Need to add a section describing shorthands. <https://github.com/w3c/css-houdini-drafts/issues/161>

Spec up URIValue, ShapeValue, StringValue, CounterValue, TimeValue, PercentageValue, FrequencyValue, VoiceValue, CustomIdentValue, TransitionTimingFunctionValue. What is attr( ) in other specs? <https://github.com/w3c/css-houdini-drafts/issues/158>

Do we want to make a generic PairValue and QuadValue, or have more specific classes for background-size, border-image-repeat, border-radius-*, border-image-outset, border-image-width? <https://github.com/w3c/css-houdini-drafts/issues/155>

BorderImageSliceValue represents [ | ]{1,4} && fill? <https://github.com/w3c/css-houdini-drafts/issues/153>

What do we do for play-during, which takes [ mix || repeat ]? <https://github.com/w3c/css-houdini-drafts/issues/152>

Do we want a pair type for things like quotes which take [ ] <https://github.com/w3c/css-houdini-drafts/issues/150>

Do we want a font value class? How about a FontWeightValue (for 100, 200 etc) class? <https://github.com/w3c/css-houdini-drafts/issues/142>

Add more CSS3 properties. This table currently only contains CSS2.1 properties and CSS3 properties alphabetically to border-radius. <https://github.com/w3c/css-houdini-drafts/issues/144>

Need an array-buffer custom property type. <https://github.com/w3c/css-houdini-drafts/issues/136>

consider using properties in addition to get/set. <https://github.com/w3c/css-houdini-drafts/issues/310>

## Conformance

### Document conventions

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

Advisements are normative sections styled to evoke special attention and are set apart from other normative text with `<strong class="advisement">`, like this: UAs MUST provide an accessible alternative.

### Conformance classes

Conformance to this specification is defined for three conformance classes:

style sheet
  A CSS style sheet.
renderer
  A UA that interprets the semantics of a style sheet and renders documents that use them.
authoring tool
  A UA that writes a style sheet.

A style sheet is conformant to this specification if all of its statements that use syntax defined in this module are valid according to the generic CSS grammar and the individual grammars of each feature defined in this module.

A renderer is conformant to this specification if, in addition to interpreting the style sheet as defined by the appropriate specifications, it supports all the features defined by this specification by parsing them correctly and rendering the document accordingly. However, the inability of a UA to correctly render a document due to limitations of the device does not make the UA non-conformant. (For example, a UA is not required to render color on a monochrome monitor.)

An authoring tool is conformant to this specification if it writes style sheets that are syntactically correct according to the generic CSS grammar and the individual grammars of each feature in this module, and meet all other conformance requirements of style sheets as described in this module.

### Partial implementations

So that authors can exploit the forward-compatible parsing rules to assign fallback values, CSS renderers must treat as invalid (and ignore as appropriate) any at-rules, properties, property values, keywords, and other syntactic constructs for which they have no usable level of support. In particular, user agents must not selectively ignore unsupported component values and honor supported values in a single multi-value property declaration: if any value is considered invalid (as unsupported values must be), CSS requires that the entire declaration be ignored.

#### Implementations of Unstable and Proprietary Features

To avoid clashes with future stable CSS features, the CSSWG recommends following best practices for the implementation of unstable features and proprietary extensions to CSS.

### Non-experimental implementations

Once a specification reaches the Candidate Recommendation stage, non-experimental implementations are possible, and implementors should release an unprefixed implementation of any CR-level feature they can demonstrate to be correctly implemented according to spec.

To establish and maintain the interoperability of CSS across implementations, the CSS Working Group requests that non-experimental CSS renderers submit an implementation report (and, if necessary, the testcases used for that implementation report) to the W3C before releasing an unprefixed implementation of any CSS features. Testcases submitted to W3C are subject to review and correction by the CSS Working Group.

Further information on submitting testcases and implementation reports can be found from on the CSS Working Group’s website at http://www.w3.org/Style/CSS/Test/. Questions should be directed to the public-css-testsuite@w3.org mailing list.

## References

### Normative References

[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119

## Issues Index

do we want a getUsedStyleMap too? Or should it be getResolvedStyleMap? <https://github.com/w3c/css-houdini-drafts/issues/157> ↵
Need to add a section describing unions of types. <https://github.com/w3c/css-houdini-drafts/issues/163> ↵
Need to add a section describing shorthands. <https://github.com/w3c/css-houdini-drafts/issues/161> ↵
Spec up URIValue, ShapeValue, StringValue, CounterValue, TimeValue, PercentageValue, FrequencyValue, VoiceValue, CustomIdentValue, TransitionTimingFunctionValue. What is attr( ) in other specs? <https://github.com/w3c/css-houdini-drafts/issues/158> ↵
Do we want to make a generic PairValue and QuadValue, or have more specific classes for background-size, border-image-repeat, border-radius-*, border-image-outset, border-image-width? <https://github.com/w3c/css-houdini-drafts/issues/155> ↵
BorderImageSliceValue represents [ | ]{1,4} && fill? <https://github.com/w3c/css-houdini-drafts/issues/153> ↵
What do we do for play-during, which takes [ mix || repeat ]? <https://github.com/w3c/css-houdini-drafts/issues/152> ↵
Do we want a pair type for things like quotes which take [ ] <https://github.com/w3c/css-houdini-drafts/issues/150> ↵
Do we want a font value class? How about a FontWeightValue (for 100, 200 etc) class? <https://github.com/w3c/css-houdini-drafts/issues/142> ↵
Add more CSS3 properties. This table currently only contains CSS2.1 properties and CSS3 properties alphabetically to border-radius. <https://github.com/w3c/css-houdini-drafts/issues/144> ↵
Need an array-buffer custom property type. <https://github.com/w3c/css-houdini-drafts/issues/136> ↵
consider using properties in addition to get/set. <https://github.com/w3c/css-houdini-drafts/issues/310> ↵